@@ -4,15 +4,16 @@ import numpy as np
 import random
 import zipfile
 import zlib
-from flarestack.shared import k_to_flux, scale_shortener, band_mask_cache_name
+from flarestack.shared import band_mask_cache_name
 from flarestack.core.energy_pdf import EnergyPDF, read_e_pdf_dict
 from flarestack.core.time_pdf import TimePDF, read_t_pdf_dict
 from flarestack.core.spatial_pdf import SpatialPDF
-from flarestack.icecube_utils.dataset_loader import data_loader
 from flarestack.utils.catalogue_loader import calculate_source_weight
 from scipy import sparse, interpolate
 from flarestack.shared import k_to_flux
 
+logger = logging.getLogger(__name__)
+
 def read_injector_dict(inj_dict):
     """Ensures that injection dictionaries remain backwards-compatible
 
@@ -35,7 +36,7 @@ def read_injector_dict(inj_dict):
         for (old_key, new_key) in maps:
 
             if old_key in list(inj_dict.keys()):
-                logging.warning("Deprecated inj_dict key '{0}' was used. Please use '{1}' in future.".format(
+                logger.warning("Deprecated inj_dict key '{0}' was used. Please use '{1}' in future.".format(
                     old_key, new_key))
                 inj_dict[new_key] = inj_dict[old_key]
 
@@ -52,7 +53,7 @@ def read_injector_dict(inj_dict):
             inj_dict["injection_spatial_pdf"] = {}
 
         # if "injection_bkg_time_pdf" not in inj_dict.keys():
-        #     logging.warning("No 'injection_bkg_time_pdf' was specified. A 'steady' pdf will be assumed.")
+        #     logger.warning("No 'injection_bkg_time_pdf' was specified. A 'steady' pdf will be assumed.")
         #     inj_dict["injection_bkg_time_pdf"] = {"time_pdf_name": "steady"}
 
     return inj_dict
@@ -69,7 +70,7 @@ class BaseInjector:
         kwargs = read_injector_dict(kwargs)
         self.inj_kwargs = kwargs
 
-        logging.info("Initialising Injector for {0}".format(season.season_name))
+        logger.info("Initialising Injector for {0}".format(season.season_name))
         self.injection_band_mask = dict()
         self.season = season
         self.season.load_background_model()
@@ -177,7 +178,7 @@ class BaseInjector:
         "energy_pdf_name".
         """
         def decorator(subclass):
-            cls.subclasses[inj_name] = subclass
+            BaseInjector.subclasses[inj_name] = subclass
             return subclass
 
         return decorator
@@ -192,24 +193,28 @@ class BaseInjector:
 
         inj_name = inj_dict["injector_name"]
 
-        if inj_name not in cls.subclasses:
-            raise ValueError('Bad Injector name {}'.format(inj_name))
+        if inj_name not in BaseInjector.subclasses:
+            raise ValueError(f'Bad Injector name {inj_name}. '
+                             f'Available options are {BaseInjector.subclasses.keys()}')
         else:
-            return cls.subclasses[inj_name](season, sources, **inj_dict)
+            return BaseInjector.subclasses[inj_name](season, sources, **inj_dict)
 
     @staticmethod
-    def get_dec_and_omega(source):
+    def get_dec_and_omega(source, bandwidth):
         # Sets half width of band
-        dec_width = np.deg2rad(2.)
+        dec_width = np.sin(np.deg2rad(bandwidth))
 
-        # Sets a declination band 5 degrees above and below the source
-        min_dec = max(-np.pi / 2., source['dec_rad'] - dec_width)
-        max_dec = min(np.pi / 2., source['dec_rad'] + dec_width)
+        sinDec = np.sin(source['dec_rad'])
+
+        # Sets a declination band above and below the source
+        min_dec = max(-1, sinDec - dec_width)
+        max_dec = min(1., sinDec + dec_width)
         # Gives the solid angle coverage of the sky for the band
-        omega = 2. * np.pi * (np.sin(max_dec) - np.sin(min_dec))
-        return dec_width, min_dec, max_dec, omega
+        omega = 2. * np.pi * (max_dec - min_dec)
+        return np.arcsin(dec_width), np.arcsin(min_dec), np.arcsin(max_dec), omega
 
 
+@BaseInjector.register_subclass('mc_injector')
 class MCInjector(BaseInjector):
     """Core Injector Class, returns a dataset on which calculations can be
     performed. This base class is tailored for injection of MC into mock
@@ -223,12 +228,14 @@ class MCInjector(BaseInjector):
         self._mc = season.get_mc()
         BaseInjector.__init__(self, season, sources, **kwargs)
 
+        self.injection_declination_bandwidth = self.inj_kwargs.pop('injection_declination_bandwidth', 1.5)
+
         try:
             self.mc_weights = self.energy_pdf.weight_mc(self._mc)
             self.n_exp = self.calculate_n_exp()
 
         except KeyError:
-            logging.warning("No Injection Arguments. Are you unblinding?")
+            logger.warning("No Injection Arguments. Are you unblinding?")
             pass
 
     def select_mc_band(self, source):
@@ -242,7 +249,7 @@ class MCInjector(BaseInjector):
         :return: band_mask: The mask which removes events outside band
         """
 
-        dec_width, min_dec, max_dec, omega = self.get_dec_and_omega(source)
+        dec_width, min_dec, max_dec, omega = self.get_dec_and_omega(source, self.injection_declination_bandwidth)
 
         band_mask = self.get_band_mask(source, min_dec, max_dec)
 
@@ -350,8 +357,13 @@ class MCInjector(BaseInjector):
             else:
                 n_s = int(n_inj)
 
-            logging.debug("Injected {0} events with an expectation of {1:.2f} events for {2}".format(
-                n_s, n_inj if isinstance(n_inj, float) else float(n_inj[0]), source["source_name"]
+            try:
+                f_n_inj = float(n_inj[0])
+            except (TypeError, IndexError):
+                f_n_inj = float(n_inj)
+
+            logger.debug("Injected {0} events with an expectation of {1:.2f} events for {2}".format(
+                n_s, f_n_inj, source["source_name"]
             ))
 
             #  If n_s = 0, skips simulation step.
@@ -379,7 +391,6 @@ class MCInjector(BaseInjector):
 
             # Generates times for each simulated event, drawing from the
             # Injector time PDF.
-
             sim_ev["time"] = self.sig_time_pdf.simulate_times(source, n_s)
 
             # Joins the new events to the signal events
@@ -410,16 +421,16 @@ class LowMemoryInjector(MCInjector):
     def calculate_n_exp(self):
 
         cats, paths, m_index, s_index = band_mask_cache_name(
-            self.season, self.sources
+            self.season, self.sources, self.injection_declination_bandwidth
         )
         self.split_cats = cats
         self.injection_band_paths = paths
 
         if np.sum([not os.path.isfile(x) for x in self.injection_band_paths]) > 0.:
-            logging.info("No saved band masks found. These will have to be made first.")
+            logger.info("No saved band masks found. These will have to be made first.")
             self.make_injection_band_mask()
 
-        self.n_exp = np.empty((len(self.sources), 1), dtype=np.dtype(
+        self.n_exp = np.zeros((len(self.sources), 1), dtype=np.dtype(
             [('source_name', 'a30'), ('n_exp', np.float),
              ('mask_index', np.int), ("source_index", np.int)]))
 
@@ -447,7 +458,8 @@ class LowMemoryInjector(MCInjector):
             injection_band_mask = sparse.lil_matrix((len(cat),
                                                      len(self._mc)), dtype=bool)
             for i, source in enumerate(cat):
-                dec_width, min_dec, max_dec, omega = self.get_dec_and_omega(source)
+                dec_width, min_dec, max_dec, omega = self.get_dec_and_omega(source,
+                                                                            self.injection_declination_bandwidth)
                 band_mask = np.logical_and(np.greater(self._mc["trueDec"], min_dec),
                                            np.less(self._mc["trueDec"], max_dec))
                 injection_band_mask[i, :] = band_mask
@@ -456,12 +468,13 @@ class LowMemoryInjector(MCInjector):
 
             del injection_band_mask
 
-            logging.info(f"Saving to {path}")
+            logger.info(f"Saving to {path}")
 
     def load_band_mask(self, index):
         path = self.injection_band_paths[index]
-        # logging.debug(f'type(band_mask_cache) = {type(self.band_mask_cache)}')
+        # logger.debug(f'type(band_mask_cache) = {type(self.band_mask_cache)}')
         del self.band_mask_cache
+        logger.debug(f'loading bandmask from {path}')
         self.band_mask_cache = sparse.load_npz(path)
         self.band_mask_index = index
         # return sparse.load_npz(path)
@@ -469,9 +482,12 @@ class LowMemoryInjector(MCInjector):
     def get_band_mask(self, source, min_dec, max_dec):
 
         entry = self.get_n_exp_single(source)
+        if len(entry) != 1:
+            raise ValueError(f"Length of found entries for {source['source_name']} "
+                             f"is {len(entry)} but should be 1!")
         mask_index = entry["mask_index"]
 
-        if not np.logical_and(self.band_mask_cache is not None,
+        if not np.logical_and(not isinstance(self.band_mask_cache, type(None)),
                               self.band_mask_index == mask_index):
             try:
                 self.load_band_mask(mask_index[0])
@@ -488,6 +504,7 @@ class LowMemoryInjector(MCInjector):
         )[0]
 
 
+@MCInjector.register_subclass("effective_area_injector")
 class EffectiveAreaInjector(BaseInjector):
     """Class for injecting signal events by relying on effective areas rather
     than pre-existing Monte Carlo simulation. This Injector should be used
@@ -530,7 +547,7 @@ class EffectiveAreaInjector(BaseInjector):
             else:
                 n_s = int(n_inj)
 
-            logging.debug("Injected {0} events with an expectation of {1:.2f} events for {2}".format(
+            logger.debug("Injected {0} events with an expectation of {1:.2f} events for {2}".format(
                 n_s, n_inj if isinstance(n_inj, float) else float(n_inj[0]), source["source_name"]
             ))
 
