@@ -13,6 +13,7 @@ from flarestack.shared import name_pickle_output_dir, \
     inj_dir_name, plot_output_dir, scale_shortener, flux_to_k
 import matplotlib.pyplot as plt
 import matplotlib.cm as cm
+from matplotlib.colors import Normalize, ListedColormap
 import matplotlib as mpl
 from flarestack.core.time_pdf import TimePDF, Box, Steady
 from flarestack.core.angular_error_modifier import BaseAngularErrorModifier
@@ -20,6 +21,8 @@ from flarestack.utils.catalogue_loader import load_catalogue, \
     calculate_source_weight
 from flarestack.utils.asimov_estimator import estimate_discovery_potential
 
+logger = logging.getLogger(__name__)
+
 def time_smear(inj):
     inj_time = inj["injection_sig_time_pdf"]
     max_length = inj_time["max_offset"] - inj_time["min_offset"]
@@ -39,18 +42,20 @@ def read_mh_dict(mh_dict):
 
     maps = [
         ("inj kwargs", "inj_dict"),
-        ("datasets", "dataset")
+        ("datasets", "dataset"),
+        ("background TS", "background_ts")
     ]
 
     for (old_key, new_key) in maps:
 
         if old_key in list(mh_dict.keys()):
-            logging.warning("Deprecated mh_dict key '{0}' was used. Please use '{1}' in future.".format(
+            logger.warning("Deprecated mh_dict key '{0}' was used. Please use '{1}' in future.".format(
                 old_key, new_key))
             mh_dict[new_key] = mh_dict[old_key]
 
     if "name" not in mh_dict.keys():
-        mh_dict["name"] = " "
+        raise KeyError("mh_dict object is missing key 'name'."
+                       "This should be the unique save path for results.")
 
     elif mh_dict["name"][-1] != "/":
         mh_dict["name"] += "/"
@@ -123,7 +128,7 @@ class MinimisationHandler(object):
 
         try:
             self.inj_seasons = mh_dict["inj_dict"]["injection_dataset"]
-            logging.debug("Using independent injection dataset.")
+            logger.debug("Using independent injection dataset.")
 
             if self.inj_seasons.keys() != self.seasons.keys():
                 raise Exception("Key mismatch between injection and llh "
@@ -145,7 +150,7 @@ class MinimisationHandler(object):
                              "selected MinimisationHandler".format(
                               self.llh_dict["llh_name"]))
         else:
-            logging.info("Using '{0}' LLH class".format(self.llh_dict["llh_name"]))
+            logger.info("Using '{0}' LLH class".format(self.llh_dict["llh_name"]))
 
         # Checks if negative n_s is specified for use, and whether this is
         # compatible with the chosen MinimisationHandler
@@ -225,11 +230,24 @@ class MinimisationHandler(object):
 
         if "fixed_scale" in list(mh_dict.keys()):
             scale_range = [mh_dict["fixed_scale"]]
+
+        # elif mh_dict.get("background_only", False):
+        #     # Only do the background trials
+        #     # In this case only n_trials background trials are performed, not 10x n_trials!
+        #     scale_range = np.array([0])
+        #
+        # elif mh_dict.get("injection_only", False):
+        #     # Only do trials with signal injection
+        #     scale = mh_dict["scale"]
+        #     steps = int(mh_dict["n_steps"])
+        #     scale_range = np.array(list(np.linspace(0., scale, steps)[1:]))
+
         else:
             scale = mh_dict["scale"]
             steps = int(mh_dict["n_steps"])
+            background_ntrials_factor = mh_dict.get('background_ntrials_factor', 10)
             scale_range = np.array(
-                [0. for _ in range(10)] +
+                [0. for _ in range(background_ntrials_factor)] +
                 list(np.linspace(0., scale, steps)[1:])
             )
 
@@ -280,8 +298,9 @@ class MinimisationHandler(object):
     def add_angular_error_modifier(self, season):
         return BaseAngularErrorModifier.create(
                 season, self.llh_dict["llh_energy_pdf"], self.floor_name,
-                self.pull_name
-            )
+                self.pull_name,
+                gamma_precision=self.llh_dict.get('gamma_precision', 'flarestack')
+        )
 
     def get_angular_error_modifier(self, season_name):
 
@@ -352,7 +371,7 @@ class FixedWeightMinimisationHandler(MinimisationHandler):
         """
 
         if self.name == " /":
-            logging.warning("No field 'name' was specified in mh_dict object. "
+            logger.warning("No field 'name' was specified in mh_dict object. "
                             "Cannot save results without a unique directory"
                             " name being specified.")
 
@@ -368,7 +387,7 @@ class FixedWeightMinimisationHandler(MinimisationHandler):
 
             file_name = os.path.join(write_dir, str(seed) + ".pkl")
 
-            logging.debug("Saving to {0}".format(file_name))
+            logger.debug("Saving to {0}".format(file_name))
 
             with open(file_name, "wb") as f:
                 Pickle.dump(results, f)
@@ -383,7 +402,6 @@ class FixedWeightMinimisationHandler(MinimisationHandler):
         else:
 
             inj_dict = self.return_injected_parameters(scale)
-            # print self.exp
 
             inj_dir = inj_dir_name(self.name)
 
@@ -394,6 +412,9 @@ class FixedWeightMinimisationHandler(MinimisationHandler):
                 pass
 
             file_name = os.path.join(inj_dir, scale_shortener(scale) + ".pkl")
+
+            logger.debug(f"Dumping Injection values to {file_name}")
+
             with open(file_name, "wb") as f:
                 Pickle.dump(inj_dict, f)
 
@@ -426,7 +447,7 @@ class FixedWeightMinimisationHandler(MinimisationHandler):
 
         best_llh = raw_f(vals)
 
-        if not (res.x[0] > 0.0) and self.negative_n_s:
+        if np.logical_and(not res.x[0] > 0.0, self.negative_n_s):
 
             bounds = list(self.bounds)
             bounds[0] = (-1000., -0.)
@@ -480,7 +501,7 @@ class FixedWeightMinimisationHandler(MinimisationHandler):
 
         mem_use = str(
             float(resource.getrusage(resource.RUSAGE_SELF).ru_maxrss) / 1.e6)
-        logging.debug('Memory usage max: {0} (Gb)'.format(mem_use))
+        logger.debug('Memory usage max: {0} (Gb)'.format(mem_use))
 
         results = {
             "TS": ts_vals,
@@ -511,7 +532,7 @@ class FixedWeightMinimisationHandler(MinimisationHandler):
         ts_vals = []
         flags = []
 
-        logging.info("Generating {0} trials!".format(n_trials))
+        logger.info("Generating {0} trials!".format(n_trials))
 
         for i in range(int(n_trials)):
 
@@ -528,20 +549,20 @@ class FixedWeightMinimisationHandler(MinimisationHandler):
             inj = self.get_injector(season)
             n_inj += np.sum(inj.n_exp["n_exp"] * scale)
 
-        logging.info("Injected with an expectation of {0} events.".format(n_inj))
+        logger.info("Injected with an expectation of {0} events.".format(n_inj))
 
-        logging.info("FIT RESULTS:")
+        logger.info("FIT RESULTS:")
 
         for (key, param) in sorted(param_vals.items()):
             if len(param) > 0:
-                logging.info("Parameter {0}: {1} {2} {3}".format(key, np.mean(param),
+                logger.info("Parameter {0}: {1} {2} {3}".format(key, np.mean(param),
                       np.median(param), np.std(param)))
-        logging.info("Test Statistic: {0} {1} {2}".format(np.mean(ts_vals),
+        logger.info("Test Statistic: {0} {1} {2}".format(np.mean(ts_vals),
               np.median(ts_vals), np.std(ts_vals)))
 
-        logging.info("FLAG STATISTICS:")
+        logger.info("FLAG STATISTICS:")
         for i in sorted(np.unique(flags)):
-            logging.info("Flag {0}:{1}".format(i, flags.count(i)))
+            logger.info("Flag {0}:{1}".format(i, flags.count(i)))
 
         results = {
             "TS": ts_vals,
@@ -550,7 +571,6 @@ class FixedWeightMinimisationHandler(MinimisationHandler):
         }
 
         self.dump_results(results, scale, seed)
-
         self.dump_injection_values(scale)
 
     def make_season_weight(self, params, season):
@@ -662,7 +682,7 @@ class FixedWeightMinimisationHandler(MinimisationHandler):
 
         return f_final
 
-    def scan_likelihood(self, scale=1., scan_2d = False):
+    def scan_likelihood(self, scale=0., scan_2d=False):
         """Generic wrapper to perform a likelihood scan a background scramble
         with an injection of signal given by scale.
 
@@ -707,7 +727,7 @@ class FixedWeightMinimisationHandler(MinimisationHandler):
 
             u_ranges.append(ur)
 
-            n_range = np.linspace(max(bound[0], -100), ur, 1e2)
+            n_range = np.linspace(float(max(bound[0], -100)), ur, int(1e2))
 
             # n_range = np.linspace(-30, 30, 1e2)
             y = []
@@ -726,35 +746,35 @@ class FixedWeightMinimisationHandler(MinimisationHandler):
             plt.xlabel(self.param_names[i])
             plt.ylabel(r"$\Delta \log(\mathcal{L}/\mathcal{L}_{0})$")
 
-            print("PARAM:", self.param_names[i])
+            logger.info(f"PARAM: {self.param_names[i]}")
             min_y = np.min(y)
-            print("Minimum value of", min_y)
 
             min_index = y.index(min_y)
             min_n = n_range[min_index]
-            print("at", min_n)
 
-            print("One Sigma interval between")
+            logger.info(f"Minimum value of {min_y} at {min_n}")
+
+            logger.info("One Sigma interval between")
 
             l_y = np.array(y[:min_index])
             try:
                 l_y = min(l_y[l_y > (min_y + 0.5)])
                 l_lim = n_range[y.index(l_y)]
-                print(l_lim)
+                logger.info(l_lim)
             except ValueError:
                 l_lim = min(n_range)
-                print("<"+str(l_lim))
+                logger.info(f"<{l_lim}")
 
-            print("and")
+            logger.info("and")
 
             u_y = np.array(y[min_index:])
             try:
                 u_y = min(u_y[u_y > (min_y + 0.5)])
                 u_lim = n_range[y.index(u_y)]
-                print(u_lim)
+                logger.info(u_lim)
             except ValueError:
                 u_lim = max(n_range)
-                print(">" + str(u_lim))
+                logger.info(f">{u_lim}")
 
             ax.axvspan(l_lim, u_lim, facecolor="grey",
                         alpha=0.2)
@@ -776,7 +796,7 @@ class FixedWeightMinimisationHandler(MinimisationHandler):
         plt.savefig(path)
         plt.close()
 
-        logging.info("Saved to {0}".format(path))
+        logger.info("Saved to {0}".format(path))
 
         # Scan 2D likelihood
 
@@ -794,7 +814,7 @@ class FixedWeightMinimisationHandler(MinimisationHandler):
 
             for j, bound in enumerate(n_s_bounds):
                 best = list(res.x)
-                plt.figure()
+                plt.figure(figsize=(5.85, 3.6154988341868854))
                 ax = plt.subplot(111)
 
                 index = np.arange(len(self.param_names))[mask][j]
@@ -802,11 +822,14 @@ class FixedWeightMinimisationHandler(MinimisationHandler):
                 plt.xlabel(r"Spectral Index ($\gamma$)")
 
                 param_name = np.array(self.param_names)[mask][j]
+                ylabel = 'n$_{\mathrm{signal}}$' if param_name == 'n_s' else param_name
+                plt.ylabel(ylabel)
 
-                plt.ylabel(param_name)
-
-                y = np.linspace(max(bound[0], -100),
-                                np.array(u_ranges)[index], 1e2)
+                y = np.linspace(
+                    float(max(bound[0], -100)),
+                    np.array(u_ranges)[index],
+                    int(1e2)
+                )
 
                 X, Y = np.meshgrid(x, y[::-1])
                 Z = []
@@ -825,9 +848,22 @@ class FixedWeightMinimisationHandler(MinimisationHandler):
 
                 levels = 0.5 * np.array([1.0, 2.0, 5.0])**2
 
-                plt.imshow(Z, aspect="auto", cmap="jet_r",
+                N = 2560
+                mmax = np.max(Z)
+                mmin = np.min(Z)
+                break_ind = int(round(N / (1 + mmax / abs(mmin))))
+                top = cm.get_cmap('gray')
+                bottom = cm.get_cmap('jet_r', N)
+                colorlist = np.empty((N, 4))
+                colorlist[break_ind:] = bottom(np.linspace(0, 1, N - break_ind))
+                colorlist[:break_ind] = top(np.linspace(1, 0, break_ind))
+                cmap = ListedColormap(colorlist)
+                norm = Normalize(vmin=mmin, vmax=mmax, clip=True)
+
+                plt.imshow(Z, aspect="auto", cmap=cmap, norm=norm,
                            extent=(x[0], x[-1], y[0], y[-1]),
                            interpolation='bilinear')
+
                 cbar = plt.colorbar()
                 CS = ax.contour(X, Y, Z, levels=levels, colors="white")
 
@@ -836,8 +872,16 @@ class FixedWeightMinimisationHandler(MinimisationHandler):
                 for l, s in zip(CS.levels, strs):
                     fmt[l] = s
 
-                ax.clabel(CS, fmt=fmt, inline=1, fontsize=10, levels=levels,
-                          color="white")
+                try:
+                    ax.clabel(CS, fmt=fmt, inline=1, fontsize=10, levels=levels,
+                              colors="white")
+                except TypeError:
+                    ax.clabel(CS, levels, fmt=fmt, inline=1, fontsize=10, #levels=levels,
+                              colors="white")
+
+                ax.set_xlim((min(x), max(x)))
+                ax.set_ylim((min(y), max(y)))
+
                 cbar.set_label(r"$\Delta \log(\mathcal{L}/\mathcal{L}_{0})$",
                                rotation=90)
 
@@ -853,16 +897,18 @@ class FixedWeightMinimisationHandler(MinimisationHandler):
 
                 plt.grid(color="white", linestyle="--", alpha=0.5)
 
-                plt.suptitle(title)
-
+                #plt.suptitle(title)
+                plt.tight_layout()
                 plt.savefig(path)
                 plt.close()
 
-                logging.info("Saved to {0}".format(path))
+                logger.info("Saved to {0}".format(path))
 
         return res_dict
 
-    def neutrino_lightcurve(self):
+    def neutrino_lightcurve(self, seed=None):
+
+        full_dataset = self.prepare_dataset(30., seed)
 
         for source in self.sources:
 
@@ -875,24 +921,22 @@ class FixedWeightMinimisationHandler(MinimisationHandler):
 
             for season in self.seasons:
 
-                time_key = season["MJD Time Key"]
-
                 # Generate a scrambled dataset, and save it to the datasets
                 # dictionary. Loads the llh for the season.
-
-                data = self.injectors[season["Name"]].create_dataset(scale=0)
-                llh = self.llhs[season["Name"]]
+                data = full_dataset[season]
+                llh = self.get_likelihood(season)
 
                 mask = llh.select_spatially_coincident_data(data, [source])
+
                 spatial_coincident_data = data[mask]
 
                 t_mask = np.logical_and(
                     np.greater(
-                        spatial_coincident_data[time_key],
-                        llh.time_pdf.sig_t0(source)),
+                        spatial_coincident_data["time"],
+                        llh.sig_time_pdf.sig_t0(source)),
                     np.less(
-                        spatial_coincident_data[time_key],
-                        llh.time_pdf.sig_t1(source))
+                        spatial_coincident_data["time"],
+                        llh.sig_time_pdf.sig_t1(source))
                 )
 
                 coincident_data = spatial_coincident_data[t_mask]
@@ -906,13 +950,16 @@ class FixedWeightMinimisationHandler(MinimisationHandler):
                 if np.sum(mask) > 0:
 
                     logE += list(10 ** (coincident_data["logE"][mask] - 3))
-                    time += list(coincident_data[time_key][mask])
+                    time += list(coincident_data["time"][mask])
                     sig += list(y)
 
-                ax0.axvline(max(llh.time_pdf.sig_t0(source), llh.time_pdf.t0),
-                            color="k", linestyle="--", alpha=0.5)
-                ax0.axvline(min(llh.time_pdf.sig_t1(source), llh.time_pdf.t1),
-                            color="k", linestyle="--", alpha=0.5)
+                if llh.sig_time_pdf.sig_t0(source) > llh.sig_time_pdf.t0:
+
+                    ax0.axvline(llh.sig_time_pdf.sig_t0(source), color="k", linestyle="--", alpha=0.5)
+
+                if llh.sig_time_pdf.sig_t1(source) < llh.sig_time_pdf.t1:
+
+                    ax0.axvline(llh.sig_time_pdf.sig_t1(source), color="k", linestyle="--", alpha=0.5)
 
             cmap = cm.get_cmap('jet')
             norm = mpl.colors.Normalize(vmin=min(logE), vmax=max(logE),
@@ -924,10 +971,14 @@ class FixedWeightMinimisationHandler(MinimisationHandler):
                 ax0.plot([x, x], [0, val], color=m.to_rgba(logE[i]))
 
             if hasattr(self, "res_dict"):
-                params = self.res_dict[source["Name"]]["Parameters"]
+                params = self.res_dict["Parameters"]
                 if len(params) > 1:
-                    ax0.axvspan(params[2], params[3], facecolor="grey",
-                                      alpha=0.2)
+                    ax0.axvspan(
+                        params[f"t_start ({source['source_name']})"],
+                        params[f"t_end ({source['source_name']})"],
+                        facecolor="grey",
+                        alpha=0.2
+                    )
             ax0.set_xlabel("Arrival Time (MJD)")
             ax0.set_ylabel("Log(Signal/Background)")
 
@@ -937,17 +988,17 @@ class FixedWeightMinimisationHandler(MinimisationHandler):
             ax1.set_ylabel("Muon Energy Proxy (TeV)")
 
             ax0.set_ylim(bottom=0)
-            plt.suptitle(source["Name"])
             # plt.tight_layout()
 
-            path = plot_output_dir(self.name) + source["Name"] + \
-                   "_neutrino_lightcurve.pdf"
+            path = f"{plot_output_dir(self.name)}neutrino_lightcurve.pdf"
 
             try:
                 os.makedirs(os.path.dirname(path))
             except OSError:
                 pass
 
+            logger.info(f"Saving to {path}")
+
             plt.savefig(path)
             plt.close()
 
@@ -1274,7 +1325,7 @@ class FlareMinimisationHandler(FixedWeightMinimisationHandler):
             # If there is are no pairs meeting this criteria, skip
 
             if len(pairs) == 0:
-                logging.debug("Continuing because no pairs")
+                logger.debug("Continuing because no pairs")
                 continue
 
             all_res = []
@@ -1449,246 +1500,6 @@ class FlareMinimisationHandler(FixedWeightMinimisationHandler):
 
         return results
 
-    # def dump_injection_values(self, scale):
-
-        # inj_dict = dict()
-        # for source in self.sources:
-        #     name = source["Name"]
-        #     n_inj = 0
-        #     for inj in self.injectors.itervalues():
-        #         try:
-        #             n_inj += inj.ref_fluxes[scale_shortener(scale)][name]
-        #
-        #         # If source not overlapping season, will not be in dict
-        #         except KeyError:
-        #             pass
-        #
-        #     default = {
-        #         "n_s": n_inj
-        #     }
-        #
-        #     if "Gamma" in self.param_names:
-        #         try:
-        #             default["Gamma"] = self.inj_kwargs["Injection Energy PDF"][
-        #                 "Gamma"]
-        #         except KeyError:
-        #             default["Gamma"] = np.nan
-
-            # if self.flare:
-            #     fs = [inj.sig_time_pdf.sig_t0(source)
-            #           for inj in self.injectors.itervalues()]
-            #     true_fs = min(fs)
-            #     fe = [inj.sig_time_pdf.sig_t1(source)
-            #           for inj in self.injectors.itervalues()]
-            #     true_fe = max(fe)
-            #
-            #     if self.time_smear:
-            #         inj_time = self.inj_kwargs["Injection Time PDF"]
-            #         offset = inj_time["Offset"]
-            #         true_fs -= offset
-            #         true_fe -= offset
-            #
-            #         min_offset = inj_time["Min Offset"]
-            #         max_offset = inj_time["Max Offset"]
-            #         med_offset = 0.5*(max_offset + min_offset)
-            #
-            #         true_fs += med_offset
-            #         true_fe += med_offset
-            #
-            #     true_l = true_fe - true_fs
-            #
-            #     sim = [
-            #         list(np.random.uniform(true_fs, true_fe,
-            #                           np.random.poisson(n_inj)))
-            #         for _ in range(1000)
-            #     ]
-            #
-            #     s = []
-            #     e = []
-            #     l = []
-            #
-            #     for data in sim:
-            #         if data != []:
-            #             s.append(min(data))
-            #             e.append(max(data))
-            #             l.append(max(data) - min(data))
-            #
-            #     if len(s) > 0:
-            #         med_s = np.median(s)
-            #         med_e = np.median(e)
-            #         med_l = np.median(l)
-            #     else:
-            #         med_s = np.nan
-            #         med_e = np.nan
-            #         med_l = np.nan
-            #
-            #     print med_s, med_e, med_l
-            #
-            #     default["Flare Start"] = med_s
-            #     default["Flare End"] = med_e
-            #     default["Flare Length"] = med_l
-
-
-    # def run(self, n_trials, scale=1.):
-    #     """Runs iterations of a flare search, and dumps results as pickle files.
-    #     For stacking of multiple sources, due to computational constraints,
-    #     each flare is minimised entirely independently. The TS values from
-    #     each flare is then summed to give an overall TS value. The results
-    #     for each source are stored separately.
-    #
-    #     :param n_trials: Number of trials to perform
-    #     :param scale: Flux scale
-    #     """
-    #
-    #     # Selects the key corresponding to time for the given IceCube dataset
-    #     # (enables use of different data samples)
-    #
-    #     seed = int(random.random() * 10 ** 8)
-    #     np.random.seed(seed)
-    #
-    #     print "Running", n_trials, "trials"
-    #
-    #     # Initialises lists for all values that will need to be stored,
-    #     # in order to verify that the minimisation is working successfuly
-    #
-    #     param_vals = {}
-    #     for key in self.param_names:
-    #         param_vals[key] = []
-    #     ts_vals = []
-    #     flags = []
-    #
-    #     print "Generating", n_trials, "trials!"
-    #
-    #     for i in range(int(n_trials)):
-    #
-    #         res_dict = self.run_trial(scale)
-    #
-    #         for (key, val) in res_dict["Parameters"].iteritems():
-    #             param_vals[key].append(val)
-    #
-    #         ts_vals.append(res_dict["TS"])
-    #         flags.append(res_dict["Flag"])
-    #
-    #     # results = {
-    #     #     "TS": [],
-    #     #     "Parameters": []
-    #     # }
-    #     #
-    #     # # for source in self.sources:
-    #     # #     results[source["Name"]] = {
-    #     # #         "TS": [],
-    #     # #         "Parameters": []
-    #     # #     }
-    #     #
-    #     # # Loop over trials
-    #     #
-    #     # for _ in range(int(n_trials)):
-    #     #
-    #     #     res_dict = self.run_trial(scale)
-    #     #
-    #     #     for j, val in enumerate(list(res_dict["Parameters"])):
-    #     #         key = self.param_names[j]
-    #     #         param_vals[key].append(val)
-    #     #
-    #     #     ts_vals.append(res_dict["TS"])
-    #     #     flags.append(res_dict["Flag"])
-    #     #
-    #     #     results["TS"].append(res_dict["TS"])
-    #
-    #     mem_use = str(
-    #         float(resource.getrusage(resource.RUSAGE_SELF).ru_maxrss) / 1.e6)
-    #     print ""
-    #     print 'Memory usage max: %s (Gb)' % mem_use
-    #
-    #     full_ts = results["TS"]
-    #
-    #     print "Combined Test Statistic:"
-    #     print np.mean(full_ts), np.median(full_ts), np.std(
-    #           full_ts)
-    #
-    #     for source in self.sources:
-    #         print "Results for", source["Name"]
-    #
-    #         combined_res = results[source["Name"]]
-    #
-    #         full_ts = combined_res["TS"]
-    #
-    #         full_params = np.array(combined_res["Parameters"])
-    #
-    #         for i, column in enumerate(full_params):
-    #             print self.param_names[i], ":", np.mean(column),\
-    #                 np.median(column), np.std(column)
-    #
-    #         print "Test Statistic", np.mean(full_ts), np.median(full_ts), \
-    #             np.std(full_ts), "\n"
-    #
-    #     self.dump_results(results, scale, seed)
-    #
-    #     self.dump_injection_values(scale)
-
-    def check_flare_background_rate(self):
-
-        results = [[] for x in self.seasons]
-        total = [[] for x in self.seasons]
-
-        for i in range(int(1000)):
-
-            # Loop over each data season
-
-            for j, season in enumerate(sorted(self.seasons)):
-
-                # Generate a scrambled dataset, and save it to the datasets
-                # dictionary. Loads the llh for the season.
-
-                data = self.injectors[season["Name"]].create_dataset(0.0)
-                llh = self.llhs[season["Name"]]
-
-                # Loops over each source in catalogue
-
-                for source in self.sources:
-
-                    # Identify spatially- and temporally-coincident data
-
-                    mask = llh.select_spatially_coincident_data(data, [source])
-                    spatial_coincident_data = data[mask]
-
-                    t_mask = np.logical_and(
-                        np.greater(spatial_coincident_data["timeMJD"],
-                                   llh.time_pdf.sig_t0(source)),
-                        np.less(spatial_coincident_data["timeMJD"],
-                                llh.time_pdf.sig_t1(source))
-                    )
-
-                    coincident_data = spatial_coincident_data[t_mask]
-                    total[j].append(len(coincident_data))
-                    # If there are events in the window...
-
-                    if len(coincident_data) > 0:
-
-                        # Identify significant events (S/B > 1)
-
-                        significant = llh.find_significant_events(
-                            coincident_data, source)
-
-                        results[j].append(len(significant))
-                    else:
-                        results[j].append(0)
-
-        for j, season in enumerate(sorted(self.seasons)):
-            res = results[j]
-            tot = total[j]
-
-            print(season["Name"],"Significant events", np.mean(res), \
-                np.median(res), np.std(res))
-            print(season["Name"], "All events", np.mean(tot), np.median(tot), \
-                np.std(tot))
-
-            llh = self.llhs[season["Name"]]
-
-            for source in self.sources:
-
-                print("Livetime", llh.time_pdf.effective_injection_time(source))
-
     @staticmethod
     def source_param_name(param, source):
         return param + " (" + str(source["source_name"]) + ")"
@@ -1748,9 +1559,22 @@ class FlareMinimisationHandler(FixedWeightMinimisationHandler):
             name = source["source_name"]
             key = self.source_param_name("n_s", source)
             n_inj = 0
-            for inj in self.injectors.values():
+
+            for season_name in self.seasons.keys():
+
                 try:
-                    n_inj += inj.ref_fluxes[scale_shortener(scale)][name]
+
+                    names = [x[0] for x in self.get_injector(season_name).n_exp["source_name"]]
+
+                    if isinstance(names[0], bytes):
+                        names = [x.decode() for x in names]
+
+                    if isinstance(name, bytes):
+                        name = name.decode()
+
+                    mask = np.array([x == name for x in names])
+
+                    n_inj += np.sum(self.get_injector(season_name).n_exp["n_exp"][mask] * scale)
 
                 # If source not overlapping season, will not be in dict
                 except KeyError:
@@ -1758,10 +1582,10 @@ class FlareMinimisationHandler(FixedWeightMinimisationHandler):
 
             inj_params[key] = n_inj
 
-            ts = min([inj.time_pdf.sig_t0(source)
-                      for inj in self.injectors.values()])
-            te = max([inj.time_pdf.sig_t1(source)
-                      for inj in self.injectors.values()])
+            ts = min([self.get_injector(season_name).sig_time_pdf.sig_t0(source)
+                      for season_name in self.seasons.keys()])
+            te = max([self.get_injector(season_name).sig_time_pdf.sig_t1(source)
+                      for season_name in self.seasons.keys()])
 
             inj_params[self.source_param_name("length", source)] = te - ts
 
@@ -1769,8 +1593,8 @@ class FlareMinimisationHandler(FixedWeightMinimisationHandler):
                 inj_params[self.source_param_name("t_start", source)] = np.nan
                 inj_params[self.source_param_name("t_end", source)] = np.nan
             else:
-                inj_params[[self.source_param_name("t_start", source)]] = ts
-                inj_params[[self.source_param_name("t_end", source)]] = te
+                inj_params[self.source_param_name("t_start", source)] = ts
+                inj_params[self.source_param_name("t_end", source)] = te
 
             for (key, val) in LLH.get_injected_parameters(
                     self.mh_dict).items():
@@ -1781,11 +1605,6 @@ class FlareMinimisationHandler(FixedWeightMinimisationHandler):
     def add_likelihood(self, season):
         return generate_dynamic_flare_class(season, self.sources, self.llh_dict)
 
-
-def g(x):
-    return x * x
-
-
 if __name__ == '__main__':
     from multiprocessing import Pool
 
@@ -1810,7 +1629,7 @@ if __name__ == '__main__':
     trials = [mh_dict["n_trials"] for _ in seeds]
     loop_args = zip(trials, scales, seeds)
 
-    logging.info("N CPUs:{0}".format(cfg.n_cpu))
+    logger.info("N CPUs:{0}".format(cfg.n_cpu))
 
     with Pool(int(cfg.n_cpu)) as p:
         p.starmap(mh.run, loop_args)
