@@ -6,9 +6,14 @@ import argparse
 from flarestack.core.minimisation import MinimisationHandler, read_mh_dict
 from multiprocessing import JoinableQueue, Process, Queue, Value
 import random
-import numpy as np
 from multiprocessing import set_start_method
-set_start_method("fork")
+
+logger = logging.getLogger(__name__)
+
+try:
+    set_start_method("fork")
+except RuntimeError:
+    pass
 
 def generate_dynamic_mh_class(mh_dict):
 
@@ -58,8 +63,8 @@ class MultiProcessor:
         # ql gets records from the queue and sends them to the handler
         ql = QueueListener(self.log_queue, handler)
         ql.start()
-        # logger = logging.getLogger()
-        # logging.getLogger().setLevel("DEBUG")
+        # logger = logger.getLogger()
+        # logger.getLogger().setLevel("DEBUG")
         # # add the handler to the logger so records from this process are handled
         # logger.addHandler(handler)
 
@@ -78,7 +83,6 @@ class MultiProcessor:
     def run_trial(self, **kwargs):
 
         qh = QueueHandler(self.log_queue)
-        logger = logging.getLogger()
         logger.addHandler(qh)
 
         mh_dict = kwargs["mh_dict"]
@@ -114,12 +118,12 @@ class MultiProcessor:
         with self.n_tasks.get_lock():
             self.n_tasks.value += n_tasks
 
-        logging.info("Added {0} trials to queue. Now processing.".format(n_tasks))
+        logger.info("Added {0} trials to queue. Now processing.".format(n_tasks))
 
         while self.n_tasks.value > 0.:
-            logging.info("{0} tasks remaining.".format(self.n_tasks.value))
+            logger.info("{0} tasks remaining.".format(self.n_tasks.value))
             time.sleep(30)
-        logging.info("Finished processing {0} tasks.".format(n_tasks))
+        logger.info("Finished processing {0} tasks.".format(n_tasks))
 
     def terminate(self):
         """ wait until queue is empty and terminate processes """
@@ -149,7 +153,7 @@ if __name__ == '__main__':
     parser.add_argument("-n", "--n_cpu", default=min(os.cpu_count()-1, 32))
     cfg = parser.parse_args()
 
-    logging.info("N CPU available {0}".format(os.cpu_count()))
+    logger.info("N CPU available {0}".format(os.cpu_count()))
 
     with open(cfg.file, "rb") as f:
         mh_dict = pickle.load(f)
