@@ -1,7 +1,6 @@
 import os
 import pickle as Pickle
 import numpy as np
-import math
 import scipy
 import scipy.stats
 import matplotlib.cm as cm
@@ -18,21 +17,30 @@ from flarestack.utils.catalogue_loader import load_catalogue
 import sys
 import logging
 
+logger = logging.getLogger(__name__)
+
+
+class OverfluctuationError(Exception):
+    pass
+
 
 class ResultsHandler(object):
 
-    def __init__(self, rh_dict):
+    def __init__(self, rh_dict, do_sens=True, do_disc=True):
 
         self.sources = load_catalogue(rh_dict["catalogue"])
 
         self.name = rh_dict["name"]
         self.mh_name = rh_dict['mh_name']
+        self.scale = rh_dict["scale"]
 
         self.results = dict()
         self.pickle_output_dir = name_pickle_output_dir(self.name)
         self.plot_dir = plot_output_dir(self.name)
         self.merged_dir = os.path.join(self.pickle_output_dir, "merged")
 
+        self.allow_extrapolation = rh_dict.get("allow_extrapolated_sensitivity", True)
+
         # Checks if the code should search for flares. By default, this is
         # not done.
         # try:
@@ -80,6 +88,11 @@ class ResultsHandler(object):
         # if cleanup:
         #     self.clean_merged_data()
 
+        # this will have the TS threshold values as keys and a tuple containing
+        # (injection scale, relative overfluctuations, error on overfluctuations)
+        # as values
+        self.overfluctuations = dict()
+
         self.sensitivity = np.nan
         self.sensitivity_err = np.nan
         self.bkg_median = np.nan
@@ -100,29 +113,69 @@ class ResultsHandler(object):
         try:
             self.merge_pickle_data()
         except FileNotFoundError:
-            logging.warning("No files found at {0}".format(self.pickle_output_dir))
+            logger.warning("No files found at {0}".format(self.pickle_output_dir))
 
         try:
             self.find_ns_scale()
         except ValueError as e:
-            logging.warning("RuntimeError for ns scale factor: \n {0}".format(e))
+            logger.warning("RuntimeError for ns scale factor: \n {0}".format(e))
 
         self.plot_bias()
 
-        try:
-            self.find_sensitivity()
-        except ValueError as e:
-            logging.warning("RuntimeError for discovery potential: \n {0}".format(e))
+        if do_sens:
+            try:
+                self.find_sensitivity()
+            except ValueError as e:
+                logger.warning("RuntimeError for discovery potential: \n {0}".format(e))
 
-        try:
-            self.find_disc_potential()
-        except RuntimeError as e:
-            logging.warning("RuntimeError for discovery potential: \n {0}".format(e))
-        except TypeError as e:
-            logging.warning("TypeError for discovery potential: \n {0}".format(e))
-        except ValueError as e:
-            logging.warning("TypeError for discovery potential: \n {0}".format(e))
+        if do_disc:
+            try:
+                self.find_disc_potential()
+            except RuntimeError as e:
+                logger.warning("RuntimeError for discovery potential: \n {0}".format(e))
+            except TypeError as e:
+                logger.warning("TypeError for discovery potential: \n {0}".format(e))
+            except ValueError as e:
+                logger.warning("TypeError for discovery potential: \n {0}".format(e))
 
+    @property
+    def scales_float(self):
+        """directly return the injected scales as floats"""
+        x = sorted(self.results.keys())
+        return sorted([float(j) for j in x])
+
+    @property
+    def scales(self):
+        """directly return the injected scales"""
+        scales = [scale_shortener(i) for i in self.scales_float]
+        return scales
+
+    @property
+    def ns(self):
+        """returns the injection scales converted to number of signal neutrinos"""
+        ns = np.array([k_to_flux(float(s)) for s in self.scales]) * self.flux_to_ns
+        return ns
+
+    @property
+    def ts_arrays(self):
+        """returns the generated test statistic distributions as arrays for each injection step"""
+        return [np.array(self.results[scale]['TS']) for scale in self.scales]
+
+    @property
+    def ns_injected(self):
+        """returns the median of the injected number of signal neutrinos for each injection step"""
+        ns_arrays = np.array([
+            np.array(
+                [np.median(self.results[scale]['Parameters'][key])
+                 for key in self.results[scale]['Parameters']
+                 if 'n_s' in key]
+            )
+            for scale in self.scales
+        ])
+
+        # In the case of fitted weights there will be a number of injected neutrinos for each source thus we have
+        # to take the sum. If this is not the case this won't do anything as ns_array will only have one entry.
+        return [sum(a) for a in ns_arrays]
 
     def astro_values(self, e_pdf_dict):
         """Function to convert the values calculated for sensitivity and
@@ -225,7 +278,7 @@ class ResultsHandler(object):
                     with open(path, "rb") as f:
                         data = Pickle.load(f)
                 except EOFError:
-                    logging.warning("Failed loading: {0}".format(path))
+                    logger.warning("Failed loading: {0}".format(path))
                     continue
                 os.remove(path)
 
@@ -239,7 +292,7 @@ class ResultsHandler(object):
                             for (param_name, params) in info.items():
                                 try: merged_data[key][param_name] += params
                                 except KeyError as m:
-                                    logging.warning('Keys [{key}][{param_name}] not found in \n {merged_data}')
+                                    logger.warning('Keys [{key}][{param_name}] not found in \n {merged_data}')
                                     raise KeyError(m)
 
             with open(merged_path, "wb") as mp:
@@ -249,32 +302,89 @@ class ResultsHandler(object):
                 self.results[scale_shortener(float(sub_dir_name))] = merged_data
 
         if len(list(self.results.keys())) == 0:
-            logging.warning("No data was found by ResultsHandler object! \n")
-            logging.warning("Tried root directory: \n {0} \n ".format(self.pickle_output_dir))
+            logger.warning("No data was found by ResultsHandler object! \n")
+            logger.warning("Tried root directory: \n {0} \n ".format(self.pickle_output_dir))
             sys.exit()
 
     def find_ns_scale(self):
-        """Find the number of neturinos corresponding to flux
-        """
-        x = sorted(self.results.keys())
-        raw_x = [scale_shortener(i) for i in sorted([float(j) for j in x])]
+        """Find the number of neutrinos corresponding to flux"""
+        # x = sorted([float(x) for x in self.results.keys()])
+
         try:
             # if weights were not fitted, number of neutrinos is stored in just one parameter
-            if not 'fit' in self.mh_name:
-                self.flux_to_ns = self.inj[raw_x[1]]["n_s"] / k_to_flux(float(x[1]))
+            if 'fit' not in self.mh_name:
+                # self.flux_to_ns = self.inj[self.scales[1]]["n_s"] / k_to_flux(float(x[1]))
+                self.flux_to_ns = self.inj[self.scales[1]]["n_s"] / k_to_flux(self.scales_float[1])
+
             # if weights were fitted, there is one n_s for each fitted source
             else:
-                sc_dict = self.inj[raw_x[1]]
-                self.flux_to_ns = sum([sc_dict[k] for k in sc_dict if 'n_s' in str(k)]) / k_to_flux(float(x[1]))
+                sc_dict = self.inj[self.scales[1]]
+                # self.flux_to_ns = sum([sc_dict[k] for k in sc_dict if 'n_s' in str(k)]) / k_to_flux(float(x[1]))
+                self.flux_to_ns = sum([sc_dict[k] for k in sc_dict if 'n_s' in str(k)]) / k_to_flux(self.scales_float[1])
 
-            logging.debug(f"Conversion ratio of flux to n_s: {self.flux_to_ns:.2f}")
+            logger.debug(f"Conversion ratio of flux to n_s: {self.flux_to_ns:.2f}")
 
         except KeyError:
-            if 'fit' in self.mh_name:
-                logging.info(f"KeyError: key \"n_s\" not found. "
-                             f"Not necessary because the minimizer {self.mh_name} is used.")
+            logger.warning(f"KeyError: key \"n_s\" not found and minimizer is {self.mh_name}!!")
+
+    def estimate_sens_disc_scale(self):
+        results = []
+
+        logger.debug('  scale   avg_sigma     avg_TS')
+        logger.debug('  ----------------------------')
+
+        for scale, ts_array in zip(self.scales_float, self.ts_arrays):
+
+            # calculate averages
+            avg_ts = ts_array.sum() / ts_array.size
+            avg_sigma = np.sqrt(avg_ts)
+
+            # error on average sigma is 1 sigma/sqrt(trials.size)
+            err_sigma = 1. / np.sqrt(ts_array.size)
+
+            # collect all sigma > 0
+            if avg_sigma >= 0:
+                logger.debug(f'  {scale:.4f}   {avg_sigma:.2f}+/-{err_sigma:.2f}  {avg_ts:.4f}')
+                results.append([scale, avg_sigma, err_sigma, avg_ts])
             else:
-                logging.warning(f"KeyError: key \"n_s\" not found and minimizer is {self.mh_name}!!")
+                pass
+
+        results = np.transpose(results)
+
+        # linear fit
+        p = np.polyfit(results[0],    # x = scale
+                       results[1],    # y = avg. sigma
+                       1,             # 1st order poly
+                       w=results[2])  # error = error on avg. sigma
+
+        # discovery threshold is 5 sigma
+        disc_scale_guess = (5 - p[1]) / p[0]
+
+        # sensitivity threshold is usually ~0.3 x discovery
+        sens_scale_guess = 0.3 * disc_scale_guess
+
+        # make a plot
+        fig, ax = plt.subplots()
+        # plot injection results
+        ax.errorbar(results[0], results[1], yerr=results[2], ls='', color='k', label='quick injections')
+        # plot the linear fit
+        xplot = np.linspace(min(results[0]), max(results[0]), 100)
+        yplot = xplot * p[0] + p[1]
+        ax.plot(xplot, yplot, marker='', label='linear fit')
+        # plot guessed scales
+        ax.axvline(disc_scale_guess, ls='--', color='red', label='DP scale guess')
+        ax.axvline(sens_scale_guess, ls='--', color='blue', label='Sens scale guess')
+        ax.set_xlabel('flux scale')
+        ax.set_ylabel('$\sigma_{mean}$')
+        ax.legend()
+        fn = os.path.join(self.plot_dir, "quick_injection_scale_guess.pdf")
+        fig.savefig(fn)
+        logger.debug(f'saved figure under {fn}')
+        plt.close()
+
+        logger.debug(f'disc scale guess: {disc_scale_guess}; sens scale guess: {sens_scale_guess}')
+
+        return disc_scale_guess, sens_scale_guess
 
     def find_sensitivity(self):
         """Uses the results of the background trials to find the median TS
@@ -286,24 +396,28 @@ class ResultsHandler(object):
         try:
             bkg_dict = self.results[scale_shortener(0.0)]
         except KeyError:
-            logging.error("No key equal to '0'")
+            logger.error("No key equal to '0'")
             return
 
         bkg_ts = bkg_dict["TS"]
+
         bkg_median = np.median(bkg_ts)
         self.bkg_median = bkg_median
 
         savepath = os.path.join(self.plot_dir, "sensitivity.pdf")
 
-        self.sensitivity, self.extrapolated_sens, self.sensitivity_err = self.find_overfluctuations(
-            bkg_median, savepath)
+        self.find_overfluctuations(bkg_median, savepath)
+        # self.sensitivity_fit(savepath, bkg_median)
+        # self.sensitivity, self.extrapolated_sens, self.sensitivity_err = self.find_overfluctuations(
+        #     bkg_median, savepath, bkg_median
+        # )
 
         msg = ""
 
         if self.extrapolated_sens:
             msg = "EXTRAPOLATED "
 
-        logging.info("{0}Sensitivity is {1:.3g}".format(msg, self.sensitivity))
+        logger.info("{0}Sensitivity is {1:.3g}".format(msg, self.sensitivity))
 
     # def set_upper_limit(self, ts_val, savepath):
     #     """Set an upper limit, based on a Test Statistic value from
@@ -337,7 +451,7 @@ class ResultsHandler(object):
     #     print "Upper limit is", "{0:.3g}".format(ul)
     #     return ul, extrapolated
 
-    def find_overfluctuations(self, ts_val, savepath):
+    def find_overfluctuations(self, ts_val, savepath=None):
         """Uses the values of injection trials to fit an 1-exponential decay
         function to the overfluctuations, allowing for calculation of the
         sensitivity. Where the injected flux was not sufficient to reach the
@@ -360,8 +474,8 @@ class ResultsHandler(object):
             ts_array = np.array(self.results[scale]["TS"])
             frac = float(len(ts_array[ts_array > ts_val])) / (float(len(
                 ts_array)))
-
-            logging.info(
+            
+            logger.info(
                 "Fraction of overfluctuations is {0:.2f} above {1:.2f} (N_trials={2}) (Scale={3})".format(
                     frac, ts_val, len(ts_array), scale
                 )
@@ -375,14 +489,16 @@ class ResultsHandler(object):
                 x_acc.append(float(scale))
                 yerr.append(1./np.sqrt(float(len(ts_array))))
 
-
                 self.make_plots(scale)
 
-        if len(np.where(np.array(y) < 0.95)[0]) < 2:
-            raise Exception(f"Not enough points with overfluctuations under 95%, lower injection scale!")
-
         x = np.array(x_acc)
+        self.overfluctuations[ts_val] = x, y, yerr
 
+        return self.sensitivity_fit(savepath, ts_val)
+
+    def sensitivity_fit(self, savepath, ts_val):
+
+        x, y, yerr = self.overfluctuations[ts_val]
         x_flux = k_to_flux(x)
 
         threshold = 0.9
@@ -407,9 +523,13 @@ class ResultsHandler(object):
         fit = k_to_flux((1./best_a) * np.log(b / (1 - threshold)))
 
         if fit > max(x_flux):
-            logging.warning("The sensitivity is beyond the range of the tested scales."
-                            "The number is probably not good.")
-            extrapolated = True
+            extrapolation_msg = "The sensitivity is beyond the range of the tested scales." \
+                                "The number is probably not good."
+            if self.allow_extrapolation:
+                logger.warning(extrapolation_msg)
+                extrapolated = True
+            else:
+                raise OverfluctuationError(extrapolation_msg)
         else:
             extrapolated = False
 
@@ -441,8 +561,15 @@ class ResultsHandler(object):
         fig.savefig(savepath)
         plt.close()
 
+        if len(np.where(np.array(y) < 0.95)[0]) < 2:
+            raise OverfluctuationError(f"Not enough points with overfluctuations under 95%, lower injection scale!")
+
         sens_err = np.array([fit - lower, upper - fit]).T[0]
 
+        self.sensitivity = fit
+        self.extrapolated_sens = extrapolated
+        self.sensitivity_err = sens_err
+
         return fit, extrapolated, sens_err
 
     def find_disc_potential(self):
@@ -452,7 +579,7 @@ class ResultsHandler(object):
         try:
             bkg_dict = self.results[scale_shortener(0.0)]
         except KeyError:
-            logging.error("No key equal to '0'")
+            logger.error("No key equal to '0'")
             return
 
         bkg_ts = bkg_dict["TS"]
@@ -474,7 +601,7 @@ class ResultsHandler(object):
             frac = float(len(ts_array[ts_array > disc_threshold])) / (
                 float(len(ts_array)))
 
-            logging.info(
+            logger.info(
                 "Fraction of overfluctuations is {0:.2f} above {1:.2f} (N_trials={2}) (Scale={3})".format(
                     frac, disc_threshold, len(ts_array), scale
                 )
@@ -484,7 +611,7 @@ class ResultsHandler(object):
             frac_25 = float(len(ts_array[ts_array > 25.])) / (
                 float(len(ts_array)))
 
-            logging.info(
+            logger.info(
                 "Fraction of overfluctuations is {0:.2f} above 25 (N_trials={1}) (Scale={2})".format(
                     frac_25, len(ts_array), scale
                 )
@@ -553,8 +680,8 @@ class ResultsHandler(object):
         if self.extrapolated_disc:
             msg = "EXTRAPOLATED "
 
-        logging.info("{0}Discovery Potential is {1:.3g}".format(msg, self.disc_potential))
-        logging.info("Discovery Potential (TS=25) is {0:.3g}".format(self.disc_potential_25))
+        logger.info("{0}Discovery Potential is {1:.3g}".format(msg, self.disc_potential))
+        logger.info("Discovery Potential (TS=25) is {0:.3g}".format(self.disc_potential_25))
 
     def noflare_plots(self, scale):
         ts_array = np.array(self.results[scale]["TS"])
@@ -573,7 +700,7 @@ class ResultsHandler(object):
 
     def ts_evolution_gif(self, n_scale_steps=None, cmap_name='winter'):
 
-        logging.debug('Making animation')
+        logger.debug('making animation')
 
         all_scales_list = list(self.results.keys())
         n_scales_all = len(all_scales_list)
@@ -596,7 +723,7 @@ class ResultsHandler(object):
         ])
 
         n_s = [sum(a) for a in ns_arrays]
-        logging.debug('Numbers of injected neutrinos: ' + str(n_s))
+        logger.debug('numbers of injected neutrinos: ' + str(n_s))
 
         norm = colors.Normalize(vmin=0, vmax=max(n_s))
         mappable = cm.ScalarMappable(norm=norm, cmap=cmap_name)
@@ -623,18 +750,18 @@ class ResultsHandler(object):
         )
 
         anim_name = os.path.join(self.plot_dir, "ts_distributions/ts_distributions_evolution.gif")
-        logging.debug(f'Saving animation under {anim_name}')
+        logger.debug('saving animation under ' + anim_name)
         anim.save(anim_name, dpi=80, writer='imagemagick')
 
     def ts_distribution_evolution(self):
 
-        logging.debug('plotting evolution of TS distribution')
+        logger.debug('plotting evolution of TS distribution')
 
         all_scales = np.array(list(self.results.keys()))
         all_scales_floats = [float(sc) for sc in all_scales]
 
-        logging.debug('all scales: ' + str(all_scales_floats))
-        logging.debug('sensitivity scale: ' + str(flux_to_k(self.sensitivity)))
+        logger.debug('all scales: ' + str(all_scales_floats))
+        logger.debug('sensitivity scale: ' + str(flux_to_k(self.sensitivity)))
 
         sens_scale = all_scales[all_scales_floats >= np.array(flux_to_k(self.sensitivity))][0]
         disc_scale = all_scales[all_scales_floats >= np.array(flux_to_k(self.disc_potential))][0]
@@ -652,7 +779,7 @@ class ResultsHandler(object):
         ])
 
         n_s = [sum(a) for a in ns_arrays]
-        logging.debug('Numbers of injected neutrinos: ' + str(n_s))
+        logger.debug('numbers of injected neutrinos: ' + str(n_s))
 
         fig, ax = plt.subplots()
 
@@ -675,8 +802,8 @@ class ResultsHandler(object):
 
         plt.tight_layout()
 
-        sn = os.path.join(self.plot_dir, "ts_distributions/ts_evolution.pdf")
-        logging.debug('Saving plot to {sn}')
+        sn = os.path.join(self.plot_dir, "ts_distributions/ts_evolution_.pdf")
+        logger.debug('saving plot to ' + sn)
         fig.savefig(sn)
 
         plt.close()
@@ -733,6 +860,8 @@ class ResultsHandler(object):
                 true = self.inj[scale][param]
                 trues.append(true)
 
+            do_ns_scale = False
+
             if "n_s" in param:
                 x = trues
                 x_label = r"$n_{injected}$" + param.replace("n_s", "")
@@ -740,6 +869,18 @@ class ResultsHandler(object):
                 x = base_x
                 x_label = base_x_label
 
+            # decide wether to plot a second x axis on the top axis indicating the number of injected neutrinos instead
+            # of the flux
+            if "gamma" in param:
+                if not isinstance(self.flux_to_ns, type(None)):
+                    do_ns_scale = True
+
+            ns_scale = ns_scale_label = None
+
+            if do_ns_scale:
+                ns_scale = self.flux_to_ns * k_to_flux(max(base_x))
+                ns_scale_label = r"Number of neutrinos"
+
             plt.scatter(x, meds, color="orange")
             plt.plot(x, meds, color="black")
             plt.plot(x, trues, linestyle="--", color="red")
@@ -749,12 +890,18 @@ class ResultsHandler(object):
             if min(trues) == 0.0:
                 ax.set_ylim(bottom=0.0)
 
+            if do_ns_scale:
+                ax2 = ax.twiny()
+                ax2.grid(0)
+                ax2.set_xlim(0., ns_scale)
+                ax2.set_xlabel(ns_scale_label)
+
             plt.xlabel(x_label)
             plt.ylabel(param)
             plt.title("Bias (" + param + ")")
 
             savepath = os.path.join(self.plot_dir, "bias_" + param + ".pdf")
-            logging.info("Saving bias plot to {0}".format(savepath))
+            logger.info("Saving bias plot to {0}".format(savepath))
 
             try:
                 os.makedirs(os.path.dirname(savepath))
