@@ -5,56 +5,13 @@ import scipy.optimize, scipy.stats
 from scipy.stats import norm
 import logging
 
+logger = logging.getLogger(__name__)
+
 raw_five_sigma = norm.cdf(5)
 n_bins = 100
 
 # Taken via Alex Stasik from Thomas Kintscher
 
-class Chi2(object):
-
-    """ A class similar to the ones from scipy.stats
-       allowing to fit left-truncated chi^2 distributions.
-    """
-
-    def __init__(self, data):
-        """ Fit the given ensemble of measurements with a chi^2 function.
-
-        `data` is a list of test statistics values.
-        `cut` defines where the distribution is truncated.
-        """
-
-        data -= min(data)
-
-        # three parameters will be fitted: dof, location, scale
-        p_start = [2., 1.]
-        p_bounds = [(0., None),  # dof > 0
-                    # (min(data), 0.),  # location < 0 for 'truncated'
-                    # effect
-                    (1e-5, 1e5)]  # shape ~ free
-
-        # define the fit function: likelihood for chi^2 distribution,
-        # plus knowledge about the amount of truncated data
-        def func(p):
-
-            dist = scipy.stats.chi2(p[0], loc=0., scale=p[1])
-            loglh = dist.logpdf(data).sum()
-            print(loglh)
-
-            return -loglh
-
-        res = scipy.optimize.minimize(func, x0=p_start, bounds=p_bounds)
-
-        if not res.success:
-            logging.warning('Chi2 fit did not converge! Result is likely garbage.')
-
-        # self._q_left = N_left / float(N_all)
-        self._cut = 0.
-        self._f = scipy.stats.chi2(res.x[0], loc=min(data), scale=res.x[1])
-        self._ks = scipy.stats.kstest(data, self._f.cdf)[0]
-        self.ndof = res.x[0]
-        self.loc = min(data)
-        self.scale = res.x[1]
-
 
 class Chi2_LeftTruncated(object):
     """ A class similar to the ones from scipy.stats
@@ -140,56 +97,6 @@ class Chi2_LeftTruncated(object):
                '\t Scale    = {0:7.2f}\n'.format(self._f.args[2]) + \
                '\t KS       = {0:7.2%}'.format(self._ks)
 
-class Double_Chi2(object):
-    """ A class similar to the ones from scipy.stats
-       allowing to fit left-truncated chi^2 distributions.
-    """
-
-    def __init__(self, data):
-        # p_start = [4., -1., 1.]
-
-        med = np.median(data)
-
-        data_right = data[data > med]
-
-        data_left = data[data <= med]
-
-        p_start = [2.]
-        p_bounds = [(0., None),  # dof > 0
-                    # (None, -0),  # location < 0 for 'truncated' effect
-                    #  shape ~ free
-                    ]
-                    # (1e-5, 1e5)]  # shape ~ free
-
-        def func(p):
-            loglh = scipy.stats.chi2(p[0], loc=med, scale=1).logpdf(
-                data_right).sum()
-            # print loglh, p
-            return -loglh
-
-        res = scipy.optimize.minimize(func, x0=p_start, bounds=p_bounds)
-
-        # print res
-
-        self._cut = 0.
-        self._res = res
-        self._f = scipy.stats.chi2(*res.x)
-        self._ks = scipy.stats.kstest(data, self._f.cdf)[0]
-        self._df = res.x[0]
-        self._loc = med
-        self._scale = 1.
-
-        def func(p):
-            loglh = scipy.stats.chi2(p[0], loc=-med, scale=1).logpdf(
-                -data_left).sum()
-            # print loglh, p
-            return -loglh
-
-        res = scipy.optimize.minimize(func, x0=p_start, bounds=p_bounds)
-
-        self.res_left = res
-        self._f_left = scipy.stats.chi2(*res.x)
-
 class Chi2_one_side(object):
 
     def __init__(self, data):
@@ -251,14 +158,31 @@ def fit_background_ts(ts_array, ts_type):
     frac_over = float(len(ts_array[mask])) / (float(len(ts_array)))
     threshold_err = 0.0
 
-    if ts_type == "Flare":
+    res = []
+    labs = []
+    cols = []
 
-        plt.hist([ts_array[mask], np.zeros(np.sum(~mask))],
-                 bins=n_bins, lw=2, histtype='step',
-                 color=['black', "grey"],
-                 label=['TS > 0', "TS <= 0"],
-                 density=True,
-                 stacked=True)
+    if np.sum(mask) > 0.:
+        res.append(ts_array[mask])
+        labs.append('TS > 0')
+        cols.append("black")
+    if np.sum(~mask) > 0.:
+        res.append(ts_array[~mask])
+        labs.append("TS <= 0")
+        cols.append("grey")
+
+    # if len(res) > 1:
+    #     res = np.array(res, dtype=object)
+    #     res.shape = (2, 1)
+
+    plt.hist(res,
+             bins=n_bins, lw=2, histtype='step',
+             color=cols,
+             label=labs,
+             density=True,
+             stacked=True)
+
+    if ts_type == "Flare":
 
         chi2 = Chi2_LeftTruncated(ts_array)
 
@@ -284,13 +208,6 @@ def fit_background_ts(ts_array, ts_type):
 
     elif ts_type == "Fit Weights":
 
-        plt.hist([ts_array[mask], np.zeros(np.sum(~mask))],
-                 bins=n_bins, lw=2, histtype='step',
-                 color=['black', "grey"],
-                 label=['TS > 0', "TS <= 0"],
-                 density=True,
-                 stacked=True)
-
         chi2 = Chi2_one_side_free(ts_array[ts_array > 0.])
 
         if chi2._res.success:
@@ -316,13 +233,6 @@ def fit_background_ts(ts_array, ts_type):
 
     elif ts_type in ["Standard", "Negative n_s"]:
 
-        plt.hist(np.array([ts_array[mask], np.zeros(np.sum(~mask))], dtype=object),
-                 bins=n_bins, lw=2, histtype='step',
-                 color=['black', "grey"],
-                 label=['TS > 0', "TS <= 0"],
-                 density=True,
-                 stacked=True)
-
         chi2 = Chi2_one_side(ts_array[ts_array > 0.])
 
         df = chi2._f.args[0]
@@ -348,20 +258,6 @@ def plot_expanded_negative(ts_array, path):
 
     med = np.median(ts_array)
 
-    chi2 = Double_Chi2(ts_array)
-
-    # frac_over = 0.5
-
-    # x_range = np.linspace(med, max(ts_array), 100)
-    #
-    # plt.plot(x_range, frac_over * chi2._f.pdf(x_range),
-    #          color="blue", label=r"$\chi^{2}$ Distribution")
-
-    # x_range = np.linspace(min(ts_array), med, 100)
-    #
-    # plt.plot(x_range, frac_over * chi2._f_left.pdf(-x_range),
-    #          color="blue")
-
     plt.yscale("log")
     plt.xlabel(r"Test Statistic ($\lambda$)")
     plt.legend(loc="upper right")
@@ -378,10 +274,11 @@ def plot_background_ts_distribution(ts_array, path, ts_type="Standard",
         pass
 
     ts_array = np.array(ts_array)
-    ts_array = ts_array[~np.isnan(ts_array)]
-
+    
     if np.sum(np.isnan(ts_array)) > 0:
-        logging.warning("TS distribution has", np.sum(np.isnan(ts_array)), "nan entries.")
+        logger.warning("TS distribution has", np.sum(np.isnan(ts_array)), "nan entries.")
+
+    ts_array = ts_array[~np.isnan(ts_array)]
 
     if np.median(ts_array) < 0.:
         plot_expanded_negative(ts_array, path)
@@ -432,7 +329,7 @@ def plot_background_ts_distribution(ts_array, path, ts_type="Standard",
             ts_val = float(ts_val[0])
 
 
-        logging.info(f"Quantifying TS: {ts_val:.2f}")
+        logger.info(f"Quantifying TS: {ts_val:.2f}")
 
         if ts_val > np.median(ts_array):
 
@@ -447,8 +344,8 @@ def plot_background_ts_distribution(ts_array, path, ts_type="Standard",
             cdf = 0.
             sig = 0.
 
-        logging.info(f"Pre-trial P-value is {1-cdf:.2E}")
-        logging.info(f"Significance is {sig:.2f} Sigma")
+        logger.info(f"Pre-trial P-value is {1-cdf:.2E}")
+        logger.info(f"Significance is {sig:.2f} Sigma")
 
         plt.axvline(ts_val, color="purple",
                     label="{:.2f}".format(ts_val) + " TS/" +
